@@ -35,9 +35,14 @@
 #include "base/logging.h"
 #include "googleurl/src/url_canon_internal.h"
 #include "googleurl/src/url_file.h"
+#include "googleurl/src/url_util_internal.h"
 
 namespace url_util {
 
+const char kFileScheme[] = "file";
+const char kFileSystemScheme[] = "filesystem";
+const char kMailtoScheme[] = "mailto";
+
 namespace {
 
 // ASCII-specific tolower.  The standard library's tolower is locale sensitive,
@@ -56,10 +61,7 @@ inline bool DoLowerCaseEqualsASCII(Iter a_begin, Iter a_end, const char* b) {
   return *b == 0;
 }
 
-const char kFileScheme[] = "file";  // Used in a number of places.
-const char kMailtoScheme[] = "mailto";
-
-const int kNumStandardURLSchemes = 7;
+const int kNumStandardURLSchemes = 8;
 const char* kStandardURLSchemes[kNumStandardURLSchemes] = {
   "http",
   "https",
@@ -68,6 +70,7 @@ const char* kStandardURLSchemes[kNumStandardURLSchemes] = {
   "gopher",
   "ws",  // WebSocket.
   "wss",  // WebSocket secure.
+  kFileSystemScheme,
 };
 
 // List of the currently installed standard schemes. This list is lazily
@@ -91,9 +94,9 @@ void InitStandardSchemes() {
 // Given a string and a range inside the string, compares it to the given
 // lower-case |compare_to| buffer.
 template<typename CHAR>
-inline bool CompareSchemeComponent(const CHAR* spec,
-                                   const url_parse::Component& component,
-                                   const char* compare_to) {
+inline bool DoCompareSchemeComponent(const CHAR* spec,
+                                     const url_parse::Component& component,
+                                     const char* compare_to) {
   if (!component.is_nonempty())
     return compare_to[0] == 0;  // When component is empty, match empty scheme.
   return LowerCaseEqualsASCII(&spec[component.begin],
@@ -138,7 +141,7 @@ bool DoFindAndCompareScheme(const CHAR* str,
   }
   if (found_scheme)
     *found_scheme = our_scheme;
-  return CompareSchemeComponent(spec, our_scheme, compare);
+  return DoCompareSchemeComponent(spec, our_scheme, compare);
 }
 
 template<typename CHAR>
@@ -169,8 +172,8 @@ bool DoCanonicalize(const CHAR* in_spec, int in_spec_len,
       url_parse::DoesBeginWindowsDriveSpec(spec, 0, spec_len)) {
     url_parse::ParseFileURL(spec, spec_len, &parsed_input);
     return url_canon::CanonicalizeFileURL(spec, spec_len, parsed_input,
-                                           charset_converter,
-                                           output, output_parsed);
+                                          charset_converter,
+                                          output, output_parsed);
   }
 #endif
 
@@ -181,12 +184,19 @@ bool DoCanonicalize(const CHAR* in_spec, int in_spec_len,
   // This is the parsed version of the input URL, we have to canonicalize it
   // before storing it in our object.
   bool success;
-  if (CompareSchemeComponent(spec, scheme, kFileScheme)) {
+  if (DoCompareSchemeComponent(spec, scheme, kFileScheme)) {
     // File URLs are special.
     url_parse::ParseFileURL(spec, spec_len, &parsed_input);
     success = url_canon::CanonicalizeFileURL(spec, spec_len, parsed_input,
-                                             charset_converter,
-                                             output, output_parsed);
+                                             charset_converter, output,
+                                             output_parsed);
+  } else if (DoCompareSchemeComponent(spec, scheme, kFileSystemScheme)) {
+    // Filesystem URLs are special.
+    url_parse::ParseFileSystemURL(spec, spec_len, &parsed_input);
+    success = url_canon::CanonicalizeFileSystemURL(spec, spec_len,
+                                                   parsed_input,
+                                                   charset_converter,
+                                                   output, output_parsed);
 
   } else if (DoIsStandard(spec, scheme)) {
     // All "normal" URLs.
@@ -195,7 +205,7 @@ bool DoCanonicalize(const CHAR* in_spec, int in_spec_len,
                                                  charset_converter,
                                                  output, output_parsed);
 
-  } else if (CompareSchemeComponent(spec, scheme, kMailtoScheme)) {
+  } else if (DoCompareSchemeComponent(spec, scheme, kMailtoScheme)) {
     // Mailto are treated like a standard url with only a scheme, path, query
     url_parse::ParseMailtoURL(spec, spec_len, &parsed_input);
     success = url_canon::CanonicalizeMailtoURL(spec, spec_len, parsed_input,
@@ -226,8 +236,17 @@ bool DoResolveRelative(const char* base_spec,
   const CHAR* relative = RemoveURLWhitespace(in_relative, in_relative_length,
                                              &whitespace_buffer,
                                              &relative_length);
+  bool base_is_authority_based = false;
+  bool base_is_hierarchical = false;
+  if (base_spec &&
+      base_parsed.scheme.is_nonempty()) {
+    int after_scheme = base_parsed.scheme.end() + 1;  // Skip past the colon.
+    int num_slashes = url_parse::CountConsecutiveSlashes(
+        base_spec, after_scheme, base_spec_len);
+    base_is_authority_based = num_slashes > 1;
+    base_is_hierarchical = num_slashes > 0;
+  }
 
-  // See if our base URL should be treated as "standard".
   bool standard_base_scheme =
       base_parsed.scheme.is_nonempty() &&
       DoIsStandard(base_spec, base_parsed.scheme);
@@ -236,17 +255,35 @@ bool DoResolveRelative(const char* base_spec,
   url_parse::Component relative_component;
   if (!url_canon::IsRelativeURL(base_spec, base_parsed,
                                 relative, relative_length,
-                                standard_base_scheme,
+                                (base_is_hierarchical || standard_base_scheme),
                                 &is_relative,
                                 &relative_component)) {
     // Error resolving.
     return false;
   }
 
-  if (is_relative) {
+  // Pretend for a moment that |base_spec| is a standard URL. Normally
+  // non-standard URLs are treated as PathURLs, but if the base has an
+  // authority we would like to preserve it.
+  if (is_relative && base_is_authority_based && !standard_base_scheme) {
+    url_parse::Parsed base_parsed_authority;
+    ParseStandardURL(base_spec, base_spec_len, &base_parsed_authority);
+    if (base_parsed_authority.host.is_nonempty()) {
+      bool did_resolve_succeed =
+          url_canon::ResolveRelativeURL(base_spec, base_parsed_authority,
+                                        false, relative,
+                                        relative_component, charset_converter,
+                                        output, output_parsed);
+      // The output_parsed is incorrect at this point (because it was built
+      // based on base_parsed_authority instead of base_parsed) and needs to be
+      // re-created.
+      ParsePathURL(output->data(), output->length(), output_parsed);
+      return did_resolve_succeed;
+    }
+  } else if (is_relative) {
     // Relative, resolve and canonicalize.
     bool file_base_scheme = base_parsed.scheme.is_nonempty() &&
-        CompareSchemeComponent(base_spec, base_parsed.scheme, kFileScheme);
+        DoCompareSchemeComponent(base_spec, base_parsed.scheme, kFileScheme);
     return url_canon::ResolveRelativeURL(base_spec, base_parsed,
                                          file_base_scheme, relative,
                                          relative_component, charset_converter,
@@ -328,15 +365,20 @@ bool DoReplaceComponents(const char* spec,
 
   // If we get here, then we know the scheme doesn't need to be replaced, so can
   // just key off the scheme in the spec to know how to do the replacements.
-  if (CompareSchemeComponent(spec, parsed.scheme, kFileScheme)) {
+  if (DoCompareSchemeComponent(spec, parsed.scheme, kFileScheme)) {
     return url_canon::ReplaceFileURL(spec, parsed, replacements,
                                      charset_converter, output, out_parsed);
   }
+  if (DoCompareSchemeComponent(spec, parsed.scheme, kFileSystemScheme)) {
+    return url_canon::ReplaceFileSystemURL(spec, parsed, replacements,
+                                           charset_converter, output,
+                                           out_parsed);
+  }
   if (DoIsStandard(spec, parsed.scheme)) {
     return url_canon::ReplaceStandardURL(spec, parsed, replacements,
                                          charset_converter, output, out_parsed);
   }
-  if (CompareSchemeComponent(spec, parsed.scheme, kMailtoScheme)) {
+  if (DoCompareSchemeComponent(spec, parsed.scheme, kMailtoScheme)) {
      return url_canon::ReplaceMailtoURL(spec, parsed, replacements,
                                         output, out_parsed);
   }
@@ -561,4 +603,16 @@ void EncodeURIComponent(const char* input, int length,
   }
 }
 
+bool CompareSchemeComponent(const char* spec,
+                            const url_parse::Component& component,
+                            const char* compare_to) {
+  return DoCompareSchemeComponent(spec, component, compare_to);
+}
+
+bool CompareSchemeComponent(const char16* spec,
+                            const url_parse::Component& component,
+                            const char* compare_to) {
+  return DoCompareSchemeComponent(spec, component, compare_to);
+}
+
 }  // namespace url_util
